/*
 * Copyright (c) 2022–2024 DTB <trinity@trinity.moe>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::{
	env::args,
	io::Error,
	os::unix::process::CommandExt,
	process::{ Command, ExitCode }
};

extern crate getopt;
use getopt::GetOpt;

extern crate strerror;
use strerror::StrError;

extern crate sysexits;
use sysexits::{ EX_UNAVAILABLE, EX_USAGE };

fn error(p: &str, n: &str, e: Error) -> ExitCode {
	eprintln!("{}: {}: {}", p, n, e.strerror());
	ExitCode::from(EX_UNAVAILABLE as u8)
}

fn usage(s: &str) -> ExitCode {
	eprintln!("Usage: {} command [argument...]", s);
	ExitCode::from(EX_USAGE as u8)
}

fn main() -> ExitCode {
	let argv = args().collect::<Vec<String>>();
	let mut optind = 1;

	/* arg0 is None if no argv[0] is meant to be given (argc will be 0),
	 * however I don't know how to do this in Rust, so the option is currently
	 * meaningless. -z will set this to None. */
	let mut arg0: Option<String> = Some(String::default());
	let mut dir = String::from("."); // $PWD for command

	while let Some(opt) = argv.getopt("C:n:z") {
		match opt.opt() {
			// Consistent with precedent; git, make, tar, etc.
			Ok("C") => dir = opt.arg().unwrap(),
			Ok("n") => arg0 = Some(opt.arg().unwrap()),
			// Ok("z") => arg0 = None,
			_ => { return usage(&argv[0]); }
		}
		optind = opt.ind();
	}

	if argv.len() - optind < 1 { return usage(&argv[0]); }

	let mut cmd = Command::new(argv[optind].clone());
	cmd.current_dir(dir.clone());

	match arg0 {
		Some(a) if a != String::default() => { cmd.arg0(a); },
		_ => () // TODO: argv[0]==NULL support
	}

	if argv.len() - optind > 1 { // there are arguments
		let cmdargs = argv
			.iter()
			.clone()
			.skip(optind + 1)
			.collect::<Vec<&String>>();
		cmd.args(cmdargs);
	}

	match cmd.spawn() {
		Ok(mut child) => match child.wait() {
			Ok(status) => match status.code() {
				Some(code) => ExitCode::from(code as u8),
				None => {
					eprintln!("{}: {}: process terminated by signal",
						argv[0], argv[2]);
					ExitCode::FAILURE
				}
			},
			Err(e) => error(&argv[0], &argv[optind], e)
		},
		Err(e) => error(&argv[0], &argv[optind], e)
	}
}
